@@ -5,6 +5,10 @@ import tornado.wsgi
 from tornado import escape, httputil
 
 
+class HurricaneWSGIException(Exception):
+    pass
+
+
 class HurricaneWSGIContainer(tornado.wsgi.WSGIContainer):
     """
     Wrapper for the tornado WSGI Container, which creates a WSGI-compatible function runnable on Tornado's
@@ -46,7 +50,7 @@ class HurricaneWSGIContainer(tornado.wsgi.WSGIContainer):
             if hasattr(app_response, "close"):
                 app_response.close()  # type: ignore
         if not data:
-            raise Exception("WSGI app did not call start_response")
+            raise HurricaneWSGIException("WSGI app did not call start_response")
 
         status_code_str, reason = data["status"].split(" ", 1)
         status_code = int(status_code_str)
